#include <stdio.h>
#include <time.h>

#ifndef TOTUTIL_TOTUTIL_TOTUTIL_TOTUTIL_TOTUTIL_TOTUTIL
#define TOTUTIL_TOTUTIL_TOTUTIL_TOTUTIL_TOTUTIL_TOTUTIL

extern "C" {
#include "eintr.h"
}

/***********************
   advanced debugging
************************/
#ifdef DEBUG
extern long DebugMessages;
#define DB_SLOT1 1	// server/client packet dumps
#define DB_SLOT2 2	// contact notifications
#define DB_SLOT3 4	// internals
#define DB_SLOT4 8	// function entries
#define DB_SLOT5 16	// errors that shouldn't happen
#define DPRINTF(f, x...) {if (f & DebugMessages) fprintf (stderr, x);}
#else
#define DPRINTF(f, x...) ;
#endif

/****************
   utilities
***************/
#define LOCALHOST 0x100007fUL

char*	getuserline (char*, char*, int);
int	filesize (char*);
char*	StrDup (char*);
char*	iStrDup (char*);
void	iStrDelete (char*);
char*	strpart (char*, char*);
char*	strcasepart (char*, char*);
void	Strncpy (char*, char*, size_t);
int	tagparse (char*, char**, int [], char**, int);
int	fputstring (char*, FILE*);
char*	fgetstring (int, FILE*);
int	connect_tcp (unsigned int, short);
int	connect_tcp (char*, short);
int	listen_tcp (short*);
int	accept_client (int);
bool	isitme (char*);
char*	network_date (time_t, char*);
int	sendfile (char*, int);
char*	Encode64 (char*, char[]);
unsigned int	peerIP (int);
unsigned int	nslookup (char*);

typedef unsigned long cryptotype;
#define CRYPTOFORMAT "%lx"

cryptotype xyc			(cryptotype,cryptotype);
cryptotype cryptorand		();
cryptotype KeyGeneration	(cryptotype,cryptotype,cryptotype*,cryptotype*);

extern "C"
unsigned long adler32		(unsigned long, char*, unsigned long);
unsigned long adler32_file	(char*);

/******************************************************************************
	zero-copy files
******************************************************************************/

class zeroCopy
{
	int fd;
   public:
	zeroCopy (char*);
	int success;
	char *d;
	int len;
	~zeroCopy ();
};

#define ZC_OK 0
#define ZC_NA 1
#define ZC_AC 2
#define ZC_FF 3
/******************************************************************************
	Interger Key Binary Tree.
******************************************************************************/

class intNode;

class intVTask
{
   public:
virtual	void Do (intNode*) = 0;
};

class intTree
{
friend	class intNode;
	intNode **FoundSlot;
	void		walk_tree	(intNode*, intVTask*);
   public:
	unsigned int	Query;
	intNode*	root;
	unsigned int	cnt;
	intTree		();
	intNode*	intFind		(unsigned int);
	void		foreach		(intVTask*);
};

class intNode
{
friend	class intTree;
   protected:
	intNode *less, *more;
	void intRemove	(intTree*);
   public:
	unsigned int Key;
	intNode (intTree*);
	intNode ();
};
/******************************************************************************
	Dynamic Binary Search Tree.
	Check dbstree.tex for info.
******************************************************************************/

#define DBS_MAGIC 33

class dbsNode;

class dbsVTask
{
   public:
virtual	void Do (dbsNode*) = 0;
};

class dbsTree
{
friend	class dbsNode;
	dbsNode **FoundSlot;
	dbsNode *gNode;
	int FoundDepth;
	void tree_to_array	(dbsNode*);
	void walk_tree		(dbsNode*, dbsVTask*);
	void walk_tree_io	(dbsNode*, dbsVTask*);
	void walk_tree_d	(dbsNode*, dbsVTask*);
	int nrt, nrl, nrm;
	void count_nodes	(dbsNode*);
	void count_guides	();
	void io_nth	(dbsNode*);
	void rio_nth	(dbsNode*);
	int dflag, tmp;
	bool tb;
   public:
	dbsNode *root;
	int nnodes;
	dbsTree		();
	void		dbsBalance	();
	dbsNode*	dbsFind		();
	dbsNode*	dbsNext		(dbsNode*);
	dbsNode*	dbsPrev		(dbsNode*);
	void		dbsRemove	(dbsNode*);
	dbsNode*	operator []	(int);
	void		copytree	(dbsVTask*);
	void		foreach		(dbsVTask*);
	void		deltree		(dbsVTask*);
	bool		iAccess, mod;
};

class dbsNode
{
friend	class dbsTree;
friend	class gtmp;
	dbsNode *less, *more;
	dbsNode*	myParent	(dbsTree*);
   protected:
	void		addself		(dbsTree*);
virtual	int		compare		(dbsNode*) = 0;
virtual	int		compare		() = 0;	/* C Comment */

   public:
	dbsNode (dbsTree*);
virtual ~dbsNode ();
};

class dbsNodeCaseStr : protected dbsNode
{
	int compare (dbsNode*);
	int compare ();
   public:
	dbsNodeCaseStr (dbsTree*);
	dbsNodeCaseStr (dbsTree*, char*);
	char *Name;
	~dbsNodeCaseStr ();

static	char *Query;
};

#define DBS_STRQUERY dbsNodeCaseStr::Query
/******************************************************************************
	argument streams
******************************************************************************/

class i_argstream
{
#define GETARG_OK	0
#define GETARG_EOF	1
#define GETARG_TOOLONG	2
#define ARGSTREAM_BUFFER 1024
#define TIMEOUT_STD	30*1000
	int timeout;
	char ibuf [ARGSTREAM_BUFFER];
	int iof, ihave;
	bool readbuf	();
   public:
	int fd;
	i_argstream	()
		{ fd = -1; timeout = TIMEOUT_STD; ihave = iof = 0; }
	i_argstream	(int, unsigned int = TIMEOUT_STD);
	bool peek	();
	int getarg	(char*, int);
	int getdata	(char*, int);
};

class o_argstream
{
	int max;
	bool release;
   public:
	char *data;
	int have;
	o_argstream	();
	void setmax	(int);
	o_argstream	(int);
	o_argstream	(char*, int);
	void putarg	(char*);
	void putarg	(char*, char*);
	void putarg	(char*, unsigned int);
	void putarg	(unsigned int);
	void putarg	(cryptotype);
	void putMsgID	(unsigned int, unsigned int);
	void putzero	();
	bool writeout	(int);
	~o_argstream	();
};
/******************************************************************************
	linked list.
******************************************************************************/

class dlistNode
{
   public:
	dlistNode *prev, *next;
};

class dlist
{
   public:
	dlist			();
	dlistNode *Start, *End;
	void addtostart		(dlistNode*);
	void addtoend		(dlistNode*);
	void addafter		(dlistNode*, dlistNode*);
	void dremove		(dlistNode*);
	void swap		(dlistNode*, dlistNode*);
	dlistNode *Next		(dlistNode*) const;
	dlistNode *Prev		(dlistNode*) const;
	unsigned int cnt;
	~dlist			();
};

inline dlistNode *dlist::Next (dlistNode *d) const
{
	return d->next;
}

inline dlistNode *dlist::Prev (dlistNode *d) const
{
	return d->prev;
}

class dlistNodeAuto : public dlistNode
{
   protected:
	void leave (dlist*);
   public:
	dlistNodeAuto	(dlist*, bool = true);
	~dlistNodeAuto	();
};

#endif
